package org.example.tiempoaemet;

import android.app.Activity;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import org.example.tiempoaemet.datos.PeriodoAEMET;

import java.util.Vector;


public class AdaptadorTiempo extends BaseAdapter {

    private static final String TAG = "AdaptadorTiempo";


    private final String PERIODO_00_06 = "00-06";
    private final String PERIODO_06_12 = "06-12";
    private final String PERIODO_12_18 = "12-18";
    private final String PERIODO_18_24 = "18-24";


    private final Activity actividad;
    private final Vector lista;


    public AdaptadorTiempo(Activity actividad, Vector lista) {
        super();
        this.actividad = actividad;
        this.lista = lista;
    }


    public View getView(int position, View convertView,   ViewGroup parent) {
        String s = null;
        LayoutInflater inflater = actividad.getLayoutInflater();
        View view = inflater.inflate(R.layout.elemento_lista, null, true);


        PeriodoAEMET periodo = (PeriodoAEMET) lista.elementAt(position);

        String meridiem = "";
        TextView textPeriodoView = (TextView) view.findViewById(R.id.periodo);
        String value = periodo.getPeriodoHoras();
        switch(value) {
            case PERIODO_00_06:
            case PERIODO_06_12:
                meridiem = "a.m";
                break;
            case PERIODO_12_18:
            case PERIODO_18_24:
                meridiem = "p.m";
                break;
            default:
                break;

        }
        textPeriodoView.setText(periodo.getPeriodoHoras()+ " "+meridiem);

        TextView textEstadoView = (TextView) view.findViewById(R.id.estado);
        String estado = periodo.getEstadoCielo();
        String nameStringResource = periodo.getExtadoCieloName(estado);
        s = MisUtilidades.getStringResourceByName(actividad, nameStringResource, this.getClass().getPackage().getName());
        textEstadoView.setText(s);

        TextView textTempView = (TextView) view.findViewById(R.id.temperatura);
        String temp = periodo.getTemperatura();


        TextView textUnidadTempView = (TextView) view.findViewById(R.id.unidadtemperatura);
        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(actividad);
        if (pref.getString("temperatura", "0").equals("0")) {
            Log.d(TAG, "ºC");
            textTempView.setText(temp);
            textUnidadTempView.setText(" ºC");
        }
        else{
            Log.d(TAG, "ºF");
            String valueF= MisUtilidades.convertCentigradeToFahrenheit(temp);
            textTempView.setText(valueF);
            textUnidadTempView.setText(" ºF");
        }


        ImageView imageView = (ImageView) view.findViewById(R.id.icono);
        String estadoCielo =  periodo.getEstadoCielo();
        switch (estadoCielo){
            case "11":
                imageView.setImageResource(R.drawable.soleado);
                break;

            case "11n":
                imageView.setImageResource(R.drawable.despejado_noche);
                break;

            case "12":
            case "12n":
            case "13":
            case "13n":
                imageView.setImageResource(R.drawable.poco_nuboso);
                break;

            case "14":
            case "14n":
            case "15":
            case "15n":
            case "16":
            case "16n":
            case "17":
            case "17n":
                imageView.setImageResource(R.drawable.nuboso);
                break;

            case "43":
            case "43n":
                imageView.setImageResource(R.drawable.intervalos_nubosos_lluvia_escasa);
                break;

            case "23":
            case "23n":
            case "24":
            case "24n":
            case "25":
            case "25n":
            case "26":
            case "26n":
            case "44":
            case "44n":
            case "45":
            case "45n":
                imageView.setImageResource(R.drawable.nuboso_lluvia);
                break;

            case "33":
            case "33n":
            case "34":
            case "34n":
            case "35":
            case "35n":
            case "36":
            case "36n":
            case "71":
            case "71n":
            case "72":
            case "72n":
            case "73":
            case "73n":
                imageView.setImageResource(R.drawable.nuboso_nieve);
                break;

            case "51":
            case "51n":
            case "52":
            case "52n":
            case "53":
            case "53n":
            case "54":
            case "54n":
            case "61":
            case "61n":
            case "62":
            case "62n":
                imageView.setImageResource(R.drawable.tormenta);
                break;

            default:
                Log.d(TAG, "Estado pendiente de clasificar: "+estadoCielo);
                imageView.setImageResource(R.drawable.unspecified);
                break;
        }

        return view;
    }



    public int getCount() {
        return lista.size();
    }
    public Object getItem(int arg0) {
        return lista.elementAt(arg0);
    }
    public long getItemId(int position) {
        return position;
    }
}
