package org.example.tiempoaemet;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.SimpleCursorAdapter;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.model.LatLng;

import org.example.tiempoaemet.bd.RelacionIneSQLite;

import java.io.IOException;
import java.util.List;
import java.util.Locale;


public class FragmentTab1 extends Fragment implements LocationListener, View.OnClickListener {

    private static final String TAG = "FragmentTab1";

    private View view;
    private Integer codigoMunicipio;
    private String nombreMunicipio, nombreProvincia;

    private EditText entradaNombreMunicipio;
    private TextView salidaNombreMunicipio;

    private Spinner spinnerProvicia;
    private Spinner spinnerMunicipio;

    private Button buttonSend;
    private Button buttonGPS;

    private RelacionIneSQLite relacionIne;


    private static final int TWO_MINUTES = 1000 * 60 * 2;

    private LocationManager manejador;
    private String proveedorGps, proveedorRed;
    private Location currentBestLocation;

    private LatLng initPosition, actualPosition;


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        view = inflater.inflate(R.layout.fragment_tab1, container, false);

        entradaNombreMunicipio = (EditText) view.findViewById(R.id.entradaManual);
        salidaNombreMunicipio = (TextView) view.findViewById(R.id.salidaManual);

        spinnerProvicia = (Spinner) view.findViewById(R.id.spinner_provincia);
        spinnerMunicipio = (Spinner) view.findViewById(R.id.spinner_municipio);
        spinnerMunicipio.setEnabled(false);

        buttonSend = (Button) view.findViewById(R.id.buttonSend);
        buttonSend.setOnClickListener(this);
        buttonGPS = (Button) view.findViewById(R.id.buttonSendGPS);
        buttonGPS.setOnClickListener(this);

        manejador = (LocationManager) getActivity().getSystemService(Context.LOCATION_SERVICE);
        proveedorGps = LocationManager.GPS_PROVIDER;
        proveedorRed = LocationManager.NETWORK_PROVIDER;

        if ( !manejador.isProviderEnabled(LocationManager.GPS_PROVIDER ) &&
                !manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
            mostrarAlertaConfiguracion();
        }

        currentBestLocation = manejador.getLastKnownLocation(proveedorRed);

        relacionIne = new RelacionIneSQLite(getActivity());

        entradaNombreMunicipio.addTextChangedListener(new TextWatcher() {
            public void afterTextChanged(Editable s) {
                Log.d(TAG, String.valueOf(s.toString()));
                Cursor cursor = relacionIne.buscarMunicipio(entradaNombreMunicipio.getText().toString());
                if ( cursor.moveToFirst() ) {
                    codigoMunicipio = cursor.getInt(0);
                    nombreMunicipio = cursor.getString(1);
                    nombreProvincia = cursor.getString(2);
                    salidaNombreMunicipio.setText(nombreMunicipio + " (" + nombreProvincia + ")");
                } else {
                    String text = MisUtilidades.getStringResourceByName(getActivity(), "no_city_matched", this.getClass().getPackage().getName());
                    salidaNombreMunicipio.setText(text);
                }
            }

            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                //Log.d(TAG, String.valueOf(s));
            }

            public void onTextChanged(CharSequence s, int start, int before, int count) {
                //Log.d(TAG, String.valueOf(s));
            }
        });


        String[] fromColumns = new String[] {"NOMBRE"};
        int[] toViews = new int[] {R.id.nombre_provincia};
        SimpleCursorAdapter sca1 = new SimpleCursorAdapter(getActivity(),
                R.layout.elemento_ine,
                relacionIne.listado("PROVINCIA"),
                fromColumns,
                toViews,
                R.layout.custom_xml_spinner_layout);
        spinnerProvicia.setAdapter(sca1);
        spinnerProvicia.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                entradaNombreMunicipio.setText("");
                salidaNombreMunicipio.setText("");

                Object o = parent.getItemAtPosition(position);
                Cursor c = (Cursor) o;
                int value = c.getInt(0);

                String[] fromColumns = new String[]{"NOMBRE_MUN"};
                int[] toViews = new int[]{R.id.nombre_municipio};
                SimpleCursorAdapter sca2 = new SimpleCursorAdapter(getActivity(),
                        R.layout.elemento_ine,
                        relacionIne.listadoMunicipios(value),
                        fromColumns,
                        toViews,
                        0);
                spinnerMunicipio.setAdapter(sca2);
                spinnerMunicipio.setEnabled(true);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });


        spinnerMunicipio.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                Object o = parent.getItemAtPosition(position);
                Cursor c = (Cursor) o;
                int value = c.getInt(0);
                nombreMunicipio = c.getString(1);
                nombreProvincia = c.getString(2);
                salidaNombreMunicipio.setText(nombreMunicipio + " (" + nombreProvincia + ")");
                codigoMunicipio = value;
                Log.d(TAG, "CODIGO SELECCIONADO: " + value);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        return view;
    }




    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
    }



    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
    }


    /**
     * Se comprueba si sigue sin haber proveedor
     */
    @Override
    public void onStart() {
        super.onStart();
        //Toast.makeText(getActivity(), "[VERIFICANDO DE NUEVO EL PROVEEDOR GPS]"  , Toast.LENGTH_SHORT).show();
        if ( !manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            buttonGPS.setEnabled(false);
            Log.d(TAG, "Botón GPS desactivado");
            //Toast.makeText(getActivity(), "[GPS NO ACTIVADO]"  , Toast.LENGTH_SHORT).show();
        }
        else{
            buttonGPS.setEnabled(true);
            Log.d(TAG, "Botón GPS activado");
            //Toast.makeText(getActivity(), "[GPS SI ACTIVADO]"  , Toast.LENGTH_SHORT).show();
        }
    }



    public void mostrarAlertaConfiguracion(){
        String strTitle  = MisUtilidades.getStringResourceByName(getActivity(), "location_configuration", this.getClass().getPackage().getName());
        String strMsj    = MisUtilidades.getStringResourceByName(getActivity(), "choose_provider", this.getClass().getPackage().getName());
        String strConfig = MisUtilidades.getStringResourceByName(getActivity(), "configuration", this.getClass().getPackage().getName());
        String strCancel = MisUtilidades.getStringResourceByName(getActivity(), "cancel", this.getClass().getPackage().getName());

        new AlertDialog.Builder(getActivity())
                .setTitle(strTitle)
                .setMessage(strMsj)
                .setPositiveButton(strConfig,
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int boton){
                                Intent i = new Intent(
                                        Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                                startActivity(i);
                            }})
                .setNegativeButton(strCancel, null)
                .show();
    }






    /** Determines whether one Location reading is better than the current Location fix
     * @param location  The new Location that you want to evaluate
     * @param currentBestLocation  The current Location fix, to which you want to compare the new one
     */
    protected boolean isBetterLocation(Location location, Location currentBestLocation) {
        if (currentBestLocation == null) {
            // A new location is always better than no location
            return true;
        }

        // Check whether the new location fix is newer or older
        long timeDelta = location.getTime() - currentBestLocation.getTime();
        boolean isSignificantlyNewer = timeDelta > TWO_MINUTES;
        boolean isSignificantlyOlder = timeDelta < -TWO_MINUTES;
        boolean isNewer = timeDelta > 0;

        // If it's been more than two minutes since the current location, use the new location
        // because the user has likely moved
        if (isSignificantlyNewer) {
            return true;
            // If the new location is more than two minutes older, it must be worse
        } else if (isSignificantlyOlder) {
            return false;
        }

        // Check whether the new location fix is more or less accurate
        int accuracyDelta = (int) (location.getAccuracy() - currentBestLocation.getAccuracy());
        boolean isLessAccurate = accuracyDelta > 0;
        boolean isMoreAccurate = accuracyDelta < 0;
        boolean isSignificantlyLessAccurate = accuracyDelta > 200;

        // Check if the old and new location are from the same provider
        boolean isFromSameProvider = isSameProvider(location.getProvider(),
                currentBestLocation.getProvider());

        // Determine location quality using a combination of timeliness and accuracy
        if (isMoreAccurate) {
            return true;
        } else if (isNewer && !isLessAccurate) {
            return true;
        } else if (isNewer && !isSignificantlyLessAccurate && isFromSameProvider) {
            return true;
        }
        return false;
    }


    /** Checks whether two providers are the same */
    private boolean isSameProvider(String provider1, String provider2) {
        if (provider1 == null) {
            return provider2 == null;
        }
        return provider1.equals(provider2);
    }



    private void actualizarProveedores() {
        if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER,
                    20 * 1000, 5, this);
        }
        if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
            manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
                    10 * 1000, 10, this);
        }

    }


    @Override
    public void onResume() {
        super.onResume();
        actualizarProveedores();
    }

    @Override
    public void onPause() {
        super.onPause();
        manejador.removeUpdates(this);
    }




    @Override
    public void onLocationChanged(Location location) {
        if ( (initPosition==null) && (currentBestLocation!=null) ){
            //Inicializamos la posicion GPS
            initPosition = new LatLng(currentBestLocation.getLatitude(),  currentBestLocation.getLongitude());
        }
        if (isBetterLocation(location, currentBestLocation)){
            currentBestLocation = location;
        }

    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {

    }

    @Override
    public void onProviderEnabled(String provider) {

    }

    @Override
    public void onProviderDisabled(String provider) {

    }



    /**
     * Obtiene la posición actual para obtener el nombre del municipio que corresponde
     */
    public boolean getCurrentLocationCity() {
        try {
            Geocoder gcd = new Geocoder(getActivity(), Locale.getDefault());
            List<Address> addresses = gcd.getFromLocation(currentBestLocation.getLatitude(),currentBestLocation.getLongitude(),1);
            if (addresses.size() > 0) {
                Log.d(TAG, "[LOCALIZACION GPS] " + addresses.get(0).getLocality());
                nombreMunicipio = addresses.get(0).getLocality();
                Cursor cursor = relacionIne.buscarMunicipio(nombreMunicipio);
                if ( cursor.moveToFirst() ) {
                    codigoMunicipio = cursor.getInt(0);
                    nombreMunicipio = cursor.getString(1);
                    nombreProvincia = cursor.getString(2);
                    salidaNombreMunicipio.setText(nombreMunicipio + " (" + nombreProvincia + ")");
                } else {
                    //No se ha encontrado municipio
                     return false;
                }
            }

        } catch (IOException ie) {
            ie.printStackTrace();
            return false;
            //TODO: hacer algo
        } catch(Exception e) {
            e.printStackTrace();
            return false;
            //TODO: hacer algo
        }
        return true;
    }





    @Override
    public void onClick(View v) {
        boolean isOk = true;
        String city = entradaNombreMunicipio.getText().toString();
        //Toast.makeText(getActivity(), "Tiempo de "+city  , Toast.LENGTH_SHORT).show();
        int id = v.getId();
        switch(id){
            case R.id.buttonSend: //Nada, ya tenemos el codigo seleccionado
                break;

            case R.id.buttonSendGPS:    //Se obtiene el municipio donde se encuentra localizado
                isOk = getCurrentLocationCity();
                break;
            default:
                break;
        }

        if (!isOk){
            String text = MisUtilidades.getStringResourceByName(getActivity(), "no_city_matched", this.getClass().getPackage().getName());
            salidaNombreMunicipio.setText(text);
            Toast.makeText(getActivity(), text  , Toast.LENGTH_SHORT).show();
            return;
        }
        //En este punto se tiene ya el código del municipio a buscar
        Log.d(TAG,"[CODIGO MUNICIPIO A BUSCAR]"+codigoMunicipio);
        Log.d(TAG,"[MUNICIPIO A BUSCAR]"+nombreMunicipio + " (" + nombreProvincia + ")");

        Intent i = new Intent(getActivity(), PrevisionesActivity.class);
        i.putExtra("codigoMunicipio", String.valueOf(codigoMunicipio));
        i.putExtra("cityStr",nombreMunicipio + " (" + nombreProvincia + ")");
        startActivity(i);
    }



}
