package org.example.tiempoaemet;

import android.app.Activity;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;


public class MisUtilidades {





    /**
     * @return Fecha actual del sistema en formato "yyyy-MM-dd".
     * @throws Exception
     */
    public static String getCurrentTime() throws Exception {
        Date time = Calendar.getInstance().getTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String timeStamp = sdf.format(time);

        return timeStamp;
    }




    /**
     * Devuelve una lista de nodos que coincidan con el nombre indicado por parámetro.
     *
     * @param element Elemento que contiene todos los nodos a buscar.
     * @param mainNodeName Nombre del nodo a buscar
     * @return Lista de nodos que coincidan con el nombre.
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException
     * @throws Exception
     */
    public static List<Node> getNodeList(Element element, String mainNodeName)
            throws ParserConfigurationException, SAXException, IOException, Exception {
        List<Node> l = new ArrayList<Node>();
        NodeList nodeList = element.getElementsByTagName(mainNodeName);
        for (int i=0; i<nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            l.add(node);
        }
        return l;
    }






    public static String getStringResourceByName(Activity actividad, String aString, String packageName) {
        String txt = null;
        int identifier = actividad.getResources().getIdentifier(aString, "string", packageName);
        if (identifier==0){ //Caso de no encontrar identificador, mostramos uno por defecto
            identifier = actividad.getResources().getIdentifier("estado_cielo_00", "string", packageName);
        }
        return actividad.getResources().getString(identifier);
    }




    /**
     * Devuelve el objeto correspondientes al elemento buscado.
     *
     * @param obj Objeto que contiene los datos a buscar.
     * @return Elemento buscado.
     * @throws Exception
     */
    public static org.w3c.dom.Element findElement(java.lang.Object obj, String nameNode) throws Exception {
        String expression = null;
        XPathExpression xp =  null;
        Element element = null;
        Element elementFound = null;

        if (!(obj instanceof org.w3c.dom.Element))
            throw new Exception("Objet is not org.w3c.dom.Element type. Current type: "+obj.getClass().toString());
        element = (Element) obj;

        XPath xpath = XPathFactory.newInstance().newXPath();
        expression = "//*[local-name()='"+nameNode+"']";
        xp = xpath.compile(expression);

        Object node = xp.evaluate(element,XPathConstants.NODE);
        elementFound = (Element) node;

        return elementFound;
    }


    /**
     * @param node Nodo por el que se buscar� el elemento.
     * @param name Nombre del elemento que queremos buscar.
     * @return Nodo que corresponde con el nombre.
     * @throws Exception
     */
    public static Node findNode(Node node, String name) throws Exception {

        if (!node.hasChildNodes()) return null;
        NodeList childList = node.getChildNodes();
        for (int j=0;j<childList.getLength();j++) {
            Node child = childList.item(j);
            //System.out.println("@@@@@  "+child.getNodeName());
            if (child.getNodeType()==Node.ELEMENT_NODE) {
                if (child.getNodeName().equals(name)) return child;
            }
        }
        return null;

    }




    public static String convertCentigradeToFahrenheit(String valueC) {
        Float centigrados = Float.parseFloat(valueC);
        Float fahrenheit = centigrados * Float.valueOf(33.8f);

        BigDecimal bd = new BigDecimal(fahrenheit);
        bd = bd.setScale(2, RoundingMode.HALF_UP);
        float decimal = bd.floatValue();
        return String.valueOf(decimal);
    }





}
