package org.example.tiempoaemet;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import org.example.tiempoaemet.datos.PeriodoAEMET;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

public class PrevisionesActivity extends ListActivity {

    private static final String TAG = "PrevisionesActivity";

    private boolean isError404 = false;

    private final String PERIODO_00_06 = "00-06";
    private final String PERIODO_06_12 = "06-12";
    private final String PERIODO_12_18 = "12-18";
    private final String PERIODO_18_24 = "18-24";

    private final String HORA_06 = "06";
    private final String HORA_12 = "12";
    private final String HORA_18 = "18";
    private final String HORA_24 = "24";



    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.previsiones);
        Log.d(TAG,"[INIT ONCREATE PREVISIONESACTIVITY]");

        TextView textCityNameView = (TextView) this.findViewById(R.id.cityname);


        Bundle extras = getIntent().getExtras();
        String cityCode = extras.getString("codigoMunicipio");
        String cityStr = extras.getString("cityStr");
        Log.d(TAG, "PREVISION PARA EL CODIGO: " + cityCode);

        //Se asignan los periodos que nos interesa mostrar de la AEMET
        Vector<String> periodos = new Vector<String>();
        periodos.add(PERIODO_00_06);
        periodos.add(PERIODO_06_12);
        periodos.add(PERIODO_12_18);
        periodos.add(PERIODO_18_24);

        //Se asignan las horas que nos interesa mostrar de la AEMET
        Vector<String> horas = new Vector<String>();
        horas.add(HORA_06);
        horas.add(HORA_12);
        horas.add(HORA_18);
        horas.add(HORA_24);


        Document docDatosAemet = null;
        Vector<PeriodoAEMET> previsionV = new Vector<PeriodoAEMET>();
        DatosAemetTarea tarea = new DatosAemetTarea();
       try {

           Log.d(TAG, "Inicio tarea asíncrona: " + Calendar.getInstance().getTime());
           tarea.execute(cityCode);
           docDatosAemet = tarea.get(5, TimeUnit.SECONDS);
           Log.d(TAG, "Fin tarea asíncrona: " + Calendar.getInstance().getTime());


           if (docDatosAemet==null){
                   textCityNameView.setText(cityStr);      //Se muestra la ciudad buscada
                   Vector<String> voidV = new Vector<String>();
                   voidV.add(cityStr);
                   setListAdapter(new AdaptadorVoidTiempo(this, voidV));
           }
           else {   //En este punto se deberían haber obtenidos datos de AEMET

               //Se obtiene el nombre del municipio y se asigna
               Element nombreElement = MisUtilidades.findElement(docDatosAemet.getDocumentElement(), "nombre");
               Element nombreProvincia = MisUtilidades.findElement(docDatosAemet.getDocumentElement(), "provincia");
               String cityName = nombreElement.getTextContent() + " ("+ nombreProvincia.getTextContent() + ")";
               textCityNameView.setText(cityName);

               //Se obtiene la fecha actual (yyyy-mm-dd) que será el nodo <dia fecha='yyyy-mm-dd'> a buscar.
               String fechaActual = MisUtilidades.getCurrentTime();
               Log.d(TAG, "Fecha actual a consultar: " + fechaActual);

               //Se obtiene el nodo que coincide con la fecha actual
               XPathFactory xPathfactory = XPathFactory.newInstance();
               XPath xpath = xPathfactory.newXPath();
               XPathExpression expr = xpath.compile("//dia[@fecha='" + fechaActual + "']");
               Node node = (Node) expr.evaluate(docDatosAemet.getDocumentElement(), XPathConstants.NODE);
               Log.d("MAIN", "ENCONTRADO : " + (node.hasChildNodes()));
               Element diaElement = (Element) node;

               //Se obtienen todos los elementos <estado_cielo>
               List<Node> listaEstadoNode = MisUtilidades.getNodeList(diaElement, "estado_cielo");
               Log.d("MAIN", "TAMAÑO NODOS <ESTADO_CIELO>: " + listaEstadoNode.size());
               for (Node nodo : listaEstadoNode) {
                   Element estadoCieloElement = (Element) nodo;
                   String periodoStr = estadoCieloElement.getAttribute("periodo");
                   String estadoCieloStr = estadoCieloElement.getTextContent();
                   //Solo interesan algunos periodos horarios (24 horas).
                   if (periodos.contains(periodoStr)) {
                       Log.d(TAG, "[NUEVO PERIODO] : " + periodoStr + " [ESTADO_CIELO] " + estadoCieloStr);
                       PeriodoAEMET periodo = new PeriodoAEMET(periodoStr, estadoCieloStr, null);
                       previsionV.add(periodo);
                   }
               }

               PeriodoAEMET periodo = null;
               //Se obtiene el elemento <temperatura>
               Element tempElement = (Element) MisUtilidades.findNode(diaElement, "temperatura");

               List<Node> listaDatosNode = MisUtilidades.getNodeList(tempElement, "dato");
               Log.d(TAG, "TAMAÑO NODOS DATO TEMPERATURA: " + listaDatosNode.size());
               for (Node nodo : listaDatosNode) {
                   Element datoElement = (Element) nodo;
                   String horaStr = datoElement.getAttribute("hora");
                   String tempStr = datoElement.getTextContent();
                   //Solo interesan algunos las horas del periodo a mostrar.
                   if (horas.contains(horaStr)) {
                       switch (horaStr) {
                           case HORA_06:
                               periodo = (PeriodoAEMET) previsionV.get(0);
                               periodo.setTemperatura(tempStr);
                               previsionV.set(0, periodo);
                               Log.d(TAG, "[ASIGNAMOS TEMP HORA_06]: " + tempStr);
                               break;
                           case HORA_12:
                               periodo = (PeriodoAEMET) previsionV.get(1);
                               periodo.setTemperatura(tempStr);
                               previsionV.set(1, periodo);
                               Log.d(TAG, "[ASIGNAMOS TEMP HORA_12]: " + tempStr);
                               break;
                           case HORA_18:
                               periodo = (PeriodoAEMET) previsionV.get(2);
                               periodo.setTemperatura(tempStr);
                               previsionV.set(2, periodo);
                               Log.d(TAG, "[ASIGNAMOS TEMP HORA_18]: " + tempStr);
                               break;
                           case HORA_24:
                               periodo = (PeriodoAEMET) previsionV.get(3);
                               periodo.setTemperatura(tempStr);
                               previsionV.set(3, periodo);
                               Log.d(TAG, "[ASIGNAMOS TEMP HORA_24]: " + tempStr);
                               break;
                           default:
                               Log.e(TAG, "No se ha detectado ninguna temperatura");
                               break;
                       }

                   }
               }

               setListAdapter(new AdaptadorTiempo(this, previsionV));
           }


       }catch(Exception e){
           Log.e(TAG, "Exception : " + e.getMessage());
           e.printStackTrace();
       }


    }



    @Override
    protected void onListItemClick(ListView listView,
                                             View view, int position, long id) {
        super.onListItemClick(listView, view, position, id);
        Object o = getListAdapter().getItem(position);
        Toast.makeText(this, "Selección: " + Integer.toString(position) + " - " + o.toString(), Toast.LENGTH_LONG).show();
    }




    public Document obtenerFicheroTiempo(String cityCode) throws Exception {
        Document doc = null;
        String recurso = "/xml/municipios/localidad_" + cityCode + ".xml";
        String urlRecurso = "http://www.aemet.es" + recurso;
        StringBuilder responseBuilder = new StringBuilder();

        try {

            URL url = new URL(urlRecurso);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("GET");
            conn.setRequestProperty("Host", "www.aemet.es");
            conn.setRequestProperty("Content-Type", "text/xml");

            if (conn.getResponseCode() != 200) {
                isError404 = (conn.getResponseCode()==404);
                throw new RuntimeException("Failed : HTTP error code : "
                        + conn.getResponseCode());
            }

            BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream()));

            String output;
            StringBuilder xmlString = new StringBuilder();
            Log.d(TAG, "Output from Server ....");

            while ((output = br.readLine()) != null) {
                System.out.println(output);
                xmlString.append(output);
            }

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            InputSource is = new InputSource(new StringReader(xmlString.toString()));
            doc = builder.parse(is);

            conn.disconnect();

        } catch (MalformedURLException e) {
            e.printStackTrace();

        } catch (IOException e) {
            e.printStackTrace();
        }

        return doc;
    }




    class DatosAemetTarea extends AsyncTask<String, Void, Document> {

        private ProgressDialog progreso;
        private Document datosAemetElement;


        @Override protected void onPreExecute() {
            progreso = new ProgressDialog(PrevisionesActivity.this);
            progreso.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            progreso.setMessage("...");
            progreso.setCancelable(false);
            progreso.show();

       }


        @Override
        protected Document doInBackground(String... cityCode) {
            Document doc = null;
            try {

                doc = obtenerFicheroTiempo(cityCode[0]);

            }catch(Exception se){
                se.printStackTrace();
            }
            return doc;
        }



        @Override
        protected void onPostExecute(Document res) {
            progreso.dismiss();
            datosAemetElement = res;
      }



        @Override
        protected void onCancelled() {

        }



        public Document getdatosAemetElement() {
            Log.d(TAG, "retornamos datoaAemetElement");
            return datosAemetElement;
        }


    }// end-Class



}
